/** \brief Simple class for 3D-vectors.
 *
 *  This class implements a "vector" data type in 3D-space.
 *  It provides overloaded operators for some basic
 *  vector operations in cartesian coordinates (adding, subtracting,
 *  inner and outer products...).
 *
 *  Author: Jan Reucker
 */

#ifndef CVECTOR_H
#define CVECTOR_H

class CVector
{
  public:
    CVector(float px = 0.0, float py = 0.0, float pz = 0.0);
    ~CVector();
    void set(float px, float py, float pz);
    float length();
    float distance(float px, float py, float pz);
    void normalize();
    bool isZero();

    // overloaded operators
    CVector operator+ (CVector);  // Vector addition
    CVector operator- (CVector);  // Vector subtraction
    CVector operator* (CVector);  // cross (outer) product
    float   operator% (CVector);  // scalar (inner) product
    CVector operator/ (float);    // scalar division
    CVector operator* (float);    // 
  
    CVector mul (CVector);   
  
    /**
     * phi is the angle between both of the vectors and
     * this method returns ( cos(phi) )^2
     */
    float angle_cos_sqr(CVector);

    float x;  /**< x component of the vector */
    float y;  /**< y component of the vector */
    float z;  /**< z component of the vector */
  
  private:

};

#endif /* CVECTOR_H */
