#include "../global.h"
#include "../defines.h"
#include <crrc_config.h>
#include "crrc_gui_main.h"
#include "crrc_ctrlgen.h"
#include "crrc_msgbox.h"
#include "crrc_calibmap.h"
#include "crrc_mousebutton.h"
#include "crrc_scaleinput.h"
#include "crrc_joy.h"
#include "util.h"

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>

#if PORTAUDIO > 0
#include <portaudio.h>
#include "../audio_interface/tx_audio.h"  // for NUM_CHANNELS
#endif


#include "../crrc_main.h"
#include "../crrc_main_cpp.h"
#include "../mouse_kbd.h"
#include "../lib_conversions.h"

#include "../T_TX_InterfaceSerial.h"


static void CGUICtrlGeneralCallback(puObject *obj);
static void CGUIInputMethodCallback(puObject *obj);
static void CGUICalibrateButtonCallback(puObject *obj);
static void CGUIJoystickButtonCallback(puObject *obj);
static void CGUIMixerButtonCallback(puObject *obj);
static void CGUIDeviceCallback(puObject *obj);
static void CGUISpeedCallback(puObject *obj);
static void CGUIJoystickCallback(puObject *obj);
#if PORTAUDIO > 0
static void CGUIAudioInterfaceCallback(puObject *obj);
#endif

static char      *inputMethods[]    = {(char*)InputMethodStrings[0], (char*)InputMethodStrings[1],
                                       (char*)InputMethodStrings[2], (char*)InputMethodStrings[3],
                                       (char*)InputMethodStrings[4], (char*)InputMethodStrings[5],
                                       (char*)InputMethodStrings[6], (char*)InputMethodStrings[7],
                                       (char*)InputMethodStrings[8], (char*)InputMethodStrings[9],
									   (char*)InputMethodStrings[10],
                                       NULL};

// Order according to values of MOUSE and KEYBOARD!
static char      *zoomCtrl[]        = {"Keyboard", "Mouse", NULL};

static char      *radioTypes[]      = {(char*)RadioTypeStrings[0], (char*)RadioTypeStrings[1],
                                       (char*)RadioTypeStrings[2], (char*)RadioTypeStrings[3],
                                       (char*)RadioTypeStrings[4], (char*)RadioTypeStrings[5],
									   (char*)RadioTypeStrings[6],
                                       NULL};

static char *serialSpeed[] = {" 9600 / 0xF0", "19200 / 0xFF", NULL};
//static char *zhenHuaSerialSpeed[] = {"19200", NULL};

#define BUTTON_BOX_HEIGHT   (2*DLG_DEF_SPACE+DLG_DEF_BUTTON_HEIGHT)
#define COMBO_W  (150)
#define COMBO_H   (20)
#define LABEL_W  (140)


CGUICtrlGeneralDialog::CGUICtrlGeneralDialog()
            : CRRCDialog(0, 0, CRRC_DIALOG_OK)
{
  
  combo_inputMethod = new puaComboBox( LABEL_W + DLG_DEF_SPACE, 
                                      BUTTON_BOX_HEIGHT + 7*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                      LABEL_W + COMBO_W,
                                      BUTTON_BOX_HEIGHT + 7*DLG_DEF_SPACE + 5*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                      inputMethods, false);
  combo_inputMethod->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_inputMethod->setLabel("Input Method");  
  combo_inputMethod->setCurrentItem(TXInterface->inputMethod());
  combo_inputMethod->setCallback(CGUIInputMethodCallback);
  combo_inputMethod->setUserData(this);
  combo_inputMethod->reveal();
  combo_inputMethod->activate();

  combo_joyNum = new puaComboBox( LABEL_W + DLG_DEF_SPACE, 
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 LABEL_W + COMBO_W,
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 NULL, false);
  combo_joyNum->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_joyNum->setLabel("Joystick device");
  combo_joyNum->setCallback(CGUIJoystickCallback);
  combo_joyNum_list = NULL;
  nCombo_joyNum = 0;
  rebuildJoystickComboBox();

  #if PORTAUDIO > 0
  combo_audioNum = new puaComboBox( LABEL_W + DLG_DEF_SPACE, 
                                   BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                  LABEL_W + COMBO_W,
                                  BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                  NULL, false);
  combo_audioNum->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_audioNum->setLabel("Audio device");
  combo_audioNum->setCallback(CGUIAudioInterfaceCallback);
  combo_audioNum->setUserData(this);
  combo_audioNum_list = NULL;
  nCombo_audioNum = 0;
  rebuildAudioInterfaceComboBox();
  #endif

  combo_device = new puaComboBox( LABEL_W + DLG_DEF_SPACE,
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 LABEL_W + COMBO_W,
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 (char**)serialDevs, false);
  combo_device->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_device->setLabel("Serial port");
  combo_device->setCurrentItem(TXInterface->getDeviceNumber((const char**)serialDevs));
  combo_device->setCallback(CGUIDeviceCallback);
  combo_device->setUserData(this);
  combo_device->hide();

  combo_speed = new puaComboBox( LABEL_W + DLG_DEF_SPACE,
                                BUTTON_BOX_HEIGHT + 5*DLG_DEF_SPACE + 2*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                LABEL_W + COMBO_W,
                                BUTTON_BOX_HEIGHT + 5*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                serialSpeed, false);
  combo_speed->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_speed->setLabel("Baudrate / Sync");
  combo_speed->setCurrentItem(TXInterface->getDeviceSpeed());
  combo_speed->setCallback(CGUISpeedCallback);
  combo_speed->setUserData(this);
  combo_speed->hide();

  button_calibrate = new puOneShot(LABEL_W + DLG_DEF_SPACE,
                                   BUTTON_BOX_HEIGHT + 4*DLG_DEF_SPACE + 2*COMBO_H + 2*DLG_DEF_BUTTON_HEIGHT,
                                   LABEL_W + COMBO_W,
                                   BUTTON_BOX_HEIGHT + 4*DLG_DEF_SPACE + 2*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT);
  button_calibrate->setLegend("Configure...");
  button_calibrate->setUserData(this);
  button_calibrate->greyOut();
  button_calibrate->setCallback(CGUICalibrateButtonCallback);

  button_mixer = new puOneShot(LABEL_W + DLG_DEF_SPACE,
                               BUTTON_BOX_HEIGHT + 3*DLG_DEF_SPACE + 2*COMBO_H + 1*DLG_DEF_BUTTON_HEIGHT,
                               LABEL_W + COMBO_W,
                               BUTTON_BOX_HEIGHT + 3*DLG_DEF_SPACE + 2*COMBO_H + 2*DLG_DEF_BUTTON_HEIGHT);
  button_mixer->setLegend("Mixer...");
  button_mixer->setUserData(this);
  button_mixer->greyOut();
  button_mixer->setCallback(CGUIMixerButtonCallback);

  button_joybuttons = new puOneShot(LABEL_W + DLG_DEF_SPACE,
                                   BUTTON_BOX_HEIGHT + 2*DLG_DEF_SPACE + 2*COMBO_H,
                                   LABEL_W + COMBO_W,
                                   BUTTON_BOX_HEIGHT + 2*DLG_DEF_SPACE + 2*COMBO_H + DLG_DEF_BUTTON_HEIGHT);
  button_joybuttons->setLegend("Buttons...");
  button_joybuttons->setUserData(this);
  button_joybuttons->greyOut();
  button_joybuttons->setCallback(CGUIJoystickButtonCallback);

  combo_zoom = new puaComboBox( LABEL_W + DLG_DEF_SPACE,
                               BUTTON_BOX_HEIGHT + 0*DLG_DEF_SPACE + 0*COMBO_H,
                               LABEL_W + COMBO_W,
                               BUTTON_BOX_HEIGHT + 0*DLG_DEF_SPACE + 1*COMBO_H,
                               zoomCtrl, false);
  combo_zoom->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_zoom->setLabel("Zoom Control");
  combo_zoom->setCurrentItem(inputDev.zoom_control-1);
  combo_zoom->reveal();
  combo_zoom->activate();

  close();
  setSize(COMBO_W + LABEL_W + 2*DLG_DEF_SPACE, 
          3*DLG_DEF_BUTTON_HEIGHT + 5*COMBO_H + 8*DLG_DEF_SPACE + BUTTON_BOX_HEIGHT);
  setCallback(CGUICtrlGeneralCallback);
  
  // center the dialog on screen: doesn't work well, as the combo box 
  // when expanded upwards, seems to influence getABox -- but also getBBox,
  // so using that doesn't help.
  int wwidth, wheight;
  int current_width = getABox()->max[0] - getABox()->min[0];
  int current_height = getABox()->max[1] - getABox()->min[1];
  puGetWindowSize(&wwidth, &wheight);
  setPosition(wwidth/2 - current_width/2, wheight/2 - current_height/2);

  reveal();
  
  // show/hide widgets
  CGUIInputMethodCallback((puObject*)combo_inputMethod);
}


/**
 * Destroy the dialog.
 */

CGUICtrlGeneralDialog::~CGUICtrlGeneralDialog()
{
  if (combo_joyNum_list != NULL)
  {
    T_GUI_Util::freenames(combo_joyNum_list, nCombo_joyNum);
    combo_joyNum_list = NULL;
    nCombo_joyNum = 0;
  }
}


/**
 * Rebuild the list behind the joystick selection combo box
 */
void CGUICtrlGeneralDialog::rebuildJoystickComboBox()
{
  std::vector<std::string> joynames;
  
  #if TEST_WITHOUT_JOYSTICK > 0
  for (int i = 0; i < TEST_WITHOUT_JOYSTICK; i++)
  {
    char number[7];
    std::string name;
    
    snprintf(number, 6, "%2d - ", i);
    name = number;
    name += "fake joystick";
    joynames.push_back(name);
  }
  #else
  for (int i = 0; i < SDL_NumJoysticks(); i++)
  {
    char number[7];
    std::string name;
    
    snprintf(number, 6, "%2d - ", i);
    name = number;
    if (SDL_JoystickName(i) != NULL)
    {
      name += SDL_JoystickName(i);
    }
    else
    {
      name += "not available (removed?)";
      std::cerr << "CGUICtrlGeneralDialog::rebuildJoystickComboBox(): error:" << std::endl;
      std::cerr << "  Joystick " << i << " is no longer available (removed after starting CRRCsim?)." << std::endl;
    }
    joynames.push_back(name);
  }
  #endif
  
  if (combo_joyNum_list != NULL)
  {
    T_GUI_Util::freenames(combo_joyNum_list, nCombo_joyNum);
    combo_joyNum_list = NULL;
    nCombo_joyNum = 0;
  }
  combo_joyNum_list = T_GUI_Util::loadnames(joynames, nCombo_joyNum);
  combo_joyNum->newList(combo_joyNum_list);
}


/**
 * Rebuild the list behind the audio selection combo box
 */
void CGUICtrlGeneralDialog::rebuildAudioInterfaceComboBox()
{
  std::vector<std::string> audiodevs;
  
  //~ for (int i = 0; i < SDL_NumJoysticks(); i++)
  //~ {
    //~ char number[7];
    //~ std::string name;
    
    //~ snprintf(number, 6, "%2d - ", i);
    //~ name = number;
    //~ if (SDL_JoystickName(i) != NULL)
    //~ {
      //~ name += SDL_JoystickName(i);
    //~ }
    //~ else
    //~ {
      //~ name += "not available (removed?)";
      //~ std::cerr << "CGUICtrlGeneralDialog::rebuildJoystickComboBox(): error:" << std::endl;
      //~ std::cerr << "  Joystick " << i << " is no longer available (removed after starting CRRCsim?)." << std::endl;
    //~ }
    //~ joynames.push_back(name);
  //~ }

  #if PORTAUDIO > 0
  // Just in case the user decides to hotplug something while the
  // dialog is open: store all currently found devices in this
  // array to make sure that a changed ID can be detected.

  // calls to PA_Initialize() and PA_Terminate() should be harmless if
  // the lib is already initialized.
  if (Pa_Initialize() == paNoError)
  {
    const PaDeviceInfo *pdi;
    int numDevices;
    #if PORTAUDIO == 18
    numDevices = Pa_CountDevices();
    #else
    numDevices = Pa_GetDeviceCount();
    #endif
    printf(    "Number of total audio input/output devices : %d.\n", numDevices );
    for( int i = 0; i < numDevices; i++ )
    {
      pdi = Pa_GetDeviceInfo( i );
      printf("Device(%d) name  \t: %s.\n", i, pdi->name);
      printf("\tMax Inputs   = %d\n", pdi->maxInputChannels  );
      printf("\tMax Outputs  = %d\n", pdi->maxOutputChannels  );
      
      if (pdi->maxInputChannels >= NUM_CHANNELS)
      {
        T_AudioDevInfo info;

        info.sDevName = pdi->name;
        info.iMaxIn   = pdi->maxInputChannels;
        info.iMaxOut  = pdi->maxOutputChannels;
        info.id       = i;
        devinfo.push_back(info);
        audiodevs.push_back(pdi->name);
      }
    }

    
    Pa_Terminate();
  }
  else
  {
    // Initialization of PortAudio failed
  }
  #else
  audiodevs.push_back("not available");
  #endif
  
  if (combo_audioNum_list != NULL)
  {
    T_GUI_Util::freenames(combo_audioNum_list, nCombo_audioNum);
    combo_audioNum_list = NULL;
    nCombo_audioNum = 0;
  }
  combo_audioNum_list = T_GUI_Util::loadnames(audiodevs, nCombo_audioNum);
  combo_audioNum->newList(combo_audioNum_list);
}


/** \brief The dialog's callback.
 *
 */
void CGUICtrlGeneralCallback(puObject *obj)
{
  if (obj->getIntegerValue() == CRRC_DIALOG_OK)
  {    
    // Dialog left by clicking OK
    CGUICtrlGeneralDialog* dlg = (CGUICtrlGeneralDialog*)obj;
    
    inputDev.zoom_control = 1+ dlg->combo_zoom->getCurrentItem();

    // if inputMethod has changed:
    int m = dlg->combo_inputMethod->getCurrentItem();
    if (TXInterface->inputMethod() != m)
    {
      cfgfile->setAttributeOverwrite("inputMethod.method", InputMethodStrings[m]);
      std::string msg = reconfigureInputMethod();
      if (msg.length())
        Global::gui->errorMsg(msg.c_str());
    }
    
    // There is no need to restart the TXInterface if only the radio type
    // has changed.
    //~ TXInterface->setRadioType(dlg->combo_radiotype->getCurrentItem());
  }
  
  puDeleteObject(obj);
}

/**
 * Callback for changed input method. Used to show/hide other
 * widgets and check whether some method is available anyway.
 */
static void CGUIInputMethodCallback(puObject *obj)
{
  CGUICtrlGeneralDialog* dlg   = (CGUICtrlGeneralDialog*)obj->getUserData();
  int                    nItem = dlg->combo_inputMethod->getCurrentItem();
  
  if (nItem == T_TX_Interface::eIM_joystick)
  {
    // Is there a joystick available?
    if (SDL_NumJoysticks())
    {
      dlg->combo_joyNum->reveal();
    }
    else
    {
#if TEST_WITHOUT_JOYSTICK > 0
      dlg->combo_joyNum->reveal();
#else     
      Global::gui->errorMsg("No joysticks available");
      dlg->combo_inputMethod->setCurrentItem(T_TX_Interface::eIM_mouse);
      dlg->combo_joyNum->hide();
#endif
    }
  }
  else
  {
    dlg->combo_joyNum->hide();
  }
  
  #if PORTAUDIO > 0
  if (nItem == T_TX_Interface::eIM_audio)
  {
    dlg->combo_audioNum->reveal();
  }
  else
  {
    dlg->combo_audioNum->hide();
  }
  #endif
  
  // value might have been changed above, re-fetch from widget
  nItem = dlg->combo_inputMethod->getCurrentItem();

  // if inputMethod has changed:
  if (TXInterface->inputMethod() != nItem)
  {
    cfgfile->setAttributeOverwrite("inputMethod.method", InputMethodStrings[nItem]);
    std::string msg = reconfigureInputMethod();
    if (msg.length())
      Global::gui->errorMsg(msg.c_str());
  }
  
  if (TXInterface->usesMapper())
  {
    dlg->button_calibrate->activate();
  }
  else
  {
    dlg->button_calibrate->greyOut();
  }

  if (TXInterface->usesMixer())
  {
    dlg->button_mixer->activate();
  }
  else
  {
    dlg->button_mixer->greyOut();
  }
  
  if ((nItem == T_TX_Interface::eIM_joystick)
        ||
       (nItem == T_TX_Interface::eIM_mouse))
  {
    dlg->button_joybuttons->activate();
  }
  else
  {
    dlg->button_joybuttons->greyOut();
  }
  
  if (nItem == T_TX_Interface::eIM_serpic)
  {
    dlg->combo_device->setCurrentItem(TXInterface->getDeviceNumber((const char**)serialDevs));
    dlg->combo_device->reveal();
	  dlg->combo_speed->newList (serialSpeed);
    dlg->combo_speed->setCurrentItem(TXInterface->getDeviceSpeed());
    dlg->combo_speed->reveal();
  }
  else if (nItem == T_TX_Interface::eIM_zhenhua)
  {
    dlg->combo_device->setCurrentItem(TXInterface->getDeviceNumber((const char**)serialDevs));
    dlg->combo_device->reveal();
//    dlg->combo_speed->newList (zhenHuaSerialSpeed);
    dlg->combo_speed->newList (T_TX_InterfaceSerial::baudRates);
    dlg->combo_speed->setCurrentItem(TXInterface->getDeviceSpeed());
    dlg->combo_speed->reveal();
  }
  else
  {
    dlg->combo_device->hide();
    dlg->combo_speed->hide();
  }

}

/**
 * Callback activated by the Calibrate button
 */
static void CGUICalibrateButtonCallback(puObject *obj)
{
  new CGUICalibMapDialog();
}

/**
 * Callback activated by the "Buttons" button
 */
static void CGUIJoystickButtonCallback(puObject *obj)
{
  CGUICtrlGeneralDialog* dlg   = (CGUICtrlGeneralDialog*)obj->getUserData();
  int                    nItem = dlg->combo_inputMethod->getCurrentItem();

  if (nItem == T_TX_Interface::eIM_mouse)
  {
    new CGUIMouseButtonDialog();
  }
  else
  {
    if (!nJoystickDlg)
    {
      joystickDlg  = new CGUIJoystickDialog();
      nJoystickDlg = 1;
    }
  }
}


/**
 * Callback activated when a different joystick is selected
 */
static void CGUIJoystickCallback(puObject *obj)
{
  puaComboBox *combo = static_cast<puaComboBox*>(obj);
  int new_joy = combo->getCurrentItem();
  
  if (inputDev.joystick_n != new_joy)
  {
    inputDev.openJoystick(new_joy);
  }
}


/**
 * Callback activated when a different audio interface is selected
 */
#if PORTAUDIO > 0
static void CGUIAudioInterfaceCallback(puObject *obj)
{
  puaComboBox *combo = static_cast<puaComboBox*>(obj);
  CGUICtrlGeneralDialog* dlg   = (CGUICtrlGeneralDialog*)obj->getUserData();
  int new_if = combo->getCurrentItem();

  // sanity-check before accessing audio-interface-specific methods
  if (TXInterface->inputMethod() == T_TX_Interface::eIM_audio)
  {
    T_TX_InterfaceAudio *TXIntAudio = static_cast<T_TX_InterfaceAudio*>(TXInterface);
    
    T_AudioDevInfo adi = dlg->devinfo[new_if];
    TXIntAudio->setNewDevice(adi.id, adi.sDevName, adi.iMaxIn, adi.iMaxOut);
    TXIntAudio->reopen();
    TXIntAudio->putBackIntoCfg(cfgfile);
  }
}
#endif


/**
 * Callback activated by the "Mixer" button
 */
static void CGUIMixerButtonCallback(puObject *obj)
{
  new CGUIInputScaleDialog(TXInterface);
}

/**
 * Callback activated by the "device" combo box
 */
static void CGUIDeviceCallback(puObject *obj)
{
  puaComboBox *cdevice = static_cast<puaComboBox*>(obj);
  int nID = cdevice->getCurrentItem();
  
  TXInterface->setDeviceNumber(nID);
  TXInterface->setDeviceName  (serialDevs[nID]);
  
  TXInterface->putBackIntoCfg(cfgfile);
  reconfigureInputMethod();
}

/**
 * Callback activated by the "speed" combo box
 */
static void CGUISpeedCallback(puObject *obj)
{
  puaComboBox *cspeed = static_cast<puaComboBox*>(obj);

  TXInterface->setDeviceSpeed(cspeed->getCurrentItem());
  TXInterface->putBackIntoCfg(cfgfile);
  reconfigureInputMethod();
}
