// implementation of class CGUIJoystickDialog

#include "crrc_joy.h"
#include "crrc_msgbox.h"
#include "../mouse_kbd.h"
#include "../crrc_main_cpp.h"
#include "../lib_conversions.h"


CGUIJoystickDialog* joystickDlg  = (CGUIJoystickDialog*)0;
int                 nJoystickDlg = 0;

static void CGUIJoystickCallback(puObject *obj);
static void CGUIJoystickButtonCallback(puObject *obj);
static void CGUIJoystickAxisCallback(puObject *obj);

#define BUTTON_BOX_HEIGHT   (2*DLG_DEF_SPACE+DLG_DEF_BUTTON_HEIGHT)
#define BUTTON_W 200
#define BUTTON_H 25
#define COMBO_H  20
#define COMBO_W  200
#define LABEL_W  170
#define NUM_W    70


const char*  bindingsButton[] = {ButtonStrings[0], ButtonStrings[1], ButtonStrings[2], ButtonStrings[3], ButtonStrings[4], 
                                 ButtonStrings[5], ButtonStrings[6], ButtonStrings[7], NULL};

const char*  bindingsAxis[] = {AxisStrings[0], AxisStrings[1], AxisStrings[2], AxisStrings[3], AxisStrings[4], NULL};


CGUIJoystickDialog::CGUIJoystickDialog() 
            : CRRCDialog(0, 0, CRRC_DIALOG_OK)
{
  // Create Widgets
  puText* info = new puText(DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 3* DLG_DEF_SPACE + 2* COMBO_H);
  info->setLabelPlace(PUPLACE_LOWER_RIGHT);
  info->setLabel("Hit button or move axis, then\nchoose function and bind...");


  comboButton = new puaComboBox(LABEL_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 1* DLG_DEF_SPACE + 1* COMBO_H,
                               LABEL_W + COMBO_W,       BUTTON_BOX_HEIGHT + 1* DLG_DEF_SPACE + 2* COMBO_H,
                               (char**)bindingsButton, false);
  comboButton->setLabelPlace(PUPLACE_LOWER_LEFT);
  comboButton->setLabel("Press some button!");
  comboButton->setCurrentItem(0);

  buttonBindButton = new puOneShot(LABEL_W + COMBO_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 1* DLG_DEF_SPACE + 1* COMBO_H,
                                   LABEL_W + COMBO_W + DLG_DEF_SPACE + BUTTON_W, BUTTON_BOX_HEIGHT + 1* DLG_DEF_SPACE + 1* COMBO_H + BUTTON_H);
  buttonBindButton->setLegend("Bind Button");
  buttonBindButton->setCallback(CGUIJoystickButtonCallback);
  buttonBindButton->setUserData(this);
  
  comboAxis = new puaComboBox(LABEL_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 0* DLG_DEF_SPACE + 0* COMBO_H,
                             LABEL_W + COMBO_W,       BUTTON_BOX_HEIGHT + 0* DLG_DEF_SPACE + 1* COMBO_H,
                             (char**)bindingsAxis, false);
  comboAxis->setLabelPlace(PUPLACE_LOWER_LEFT);
  comboAxis->setLabel("Move some axis!");
  comboAxis->setCurrentItem(0);
  
  buttonBindAxis = new puOneShot(LABEL_W + COMBO_W + DLG_DEF_SPACE, BUTTON_BOX_HEIGHT + 0* DLG_DEF_SPACE + 0* COMBO_H,
                                 LABEL_W + COMBO_W + DLG_DEF_SPACE + BUTTON_W, BUTTON_BOX_HEIGHT + 0* DLG_DEF_SPACE + 0* COMBO_H + BUTTON_H);
  buttonBindAxis->setLegend("Bind Axis");
  buttonBindAxis->setCallback(CGUIJoystickAxisCallback);
  buttonBindAxis->setUserData(this);
      
  close();
  setSize(BUTTON_W + COMBO_W + LABEL_W + 2*DLG_DEF_SPACE,
          BUTTON_BOX_HEIGHT + 3* DLG_DEF_SPACE + 4* COMBO_H);
  setCallback(CGUIJoystickCallback);
  
  // center the dialog on screen
  int wwidth, wheight;
  int current_width = getABox()->max[0] - getABox()->min[0];
  int current_height = getABox()->max[1] - getABox()->min[1];
  puGetWindowSize(&wwidth, &wheight);
  setPosition(wwidth/2 - current_width/2, wheight*2/3 - current_height);

  nAxis   = -1;
  nButton = -1;
  
  reveal();
}


/**
 * Destroy the dialog.
 */

CGUIJoystickDialog::~CGUIJoystickDialog()
{
}

/** \brief The dialog's callback.
 *
 */
void CGUIJoystickCallback(puObject *obj)
{
  nJoystickDlg = 0;
  
  if (obj->getIntegerValue() == CRRC_DIALOG_OK)
  {
    // Dialog left by clicking OK
  }
  
  puDeleteObject(obj);  
}

/** \brief callback to bind something
 *
 */
void CGUIJoystickButtonCallback(puObject *obj)
{
  CGUIJoystickDialog* dlg = (CGUIJoystickDialog*)obj->getUserData();
  
  if (dlg->nButton >= 0)
  {
    inputDev.joystick_bind_b[dlg->nButton] = dlg->comboButton->getCurrentItem();
  }
  else
  {
    new CGUIMsgBox("Please press button first!");
  }
}

/** \brief callback to bind something
 *
 */
void CGUIJoystickAxisCallback(puObject *obj)
{
  CGUIJoystickDialog* dlg = (CGUIJoystickDialog*)obj->getUserData();
  
  if (dlg->nAxis >= 0)
  {
    inputDev.joystick_bind_x[dlg->nAxis] = dlg->comboAxis->getCurrentItem();
  }
  else
  {
    new CGUIMsgBox("Please move axis far out of center first!");
  }
}

void CGUIJoystickDialog::joystickDlgButton(SDL_JoyButtonEvent *event)
{
  int         nButton = event->button;
  
  if (event->state != SDL_PRESSED || event->button > MAXJOYBUTTON)
    return;
  
  joystickDlg->nButton = nButton;
  
  // Set label
  textButton = "Button " + itoStr(nButton, ' ', 2);
  joystickDlg->comboButton->setLabel(textButton.c_str());
  
  // Update combo box
  joystickDlg->comboButton->setCurrentItem(inputDev.joystick_bind_b[nButton]);
}
   
void CGUIJoystickDialog::joystickDlgMotion(SDL_JoyAxisEvent *event)
{
  int         nAxis = event->axis;

  // Only continue if this axis is support and if it has moved significantly out of center
  if (nAxis > MAXJOYAXIS || abs(event->value) < 10000)
    return;
  
  joystickDlg->nAxis = nAxis;
  
  // Set label
  textAxis = "Axis " + itoStr(nAxis, ' ', 2);
  joystickDlg->comboAxis->setLabel(textAxis.c_str());
  
  // Update combo box
  joystickDlg->comboAxis->setCurrentItem(inputDev.joystick_bind_x[nAxis]);
}
