// implementation of class CGUILocationDialog

#include "../global.h"
#include "../SimStateHandler.h"
#include "../crrc_scenery.h"
#include "crrc_gui_main.h"
#include "crrc_location.h"
#include "../crrc_main_cpp.h"
#include "../crrc_main.h"
#include "../windfield.h"
#include "../crrc_graphics.h"

#include <iostream>

static void CGUILocationCallback(puObject *obj);

char *locations[] = {"Cape Cod", "Davis", NULL};

#define BUTTON_BOX_HEIGHT   (2*DLG_DEF_SPACE+DLG_DEF_BUTTON_HEIGHT)

CGUILocationDialog::CGUILocationDialog() 
            : CRRCDialog(), cbox(NULL)
{
  int curLoc = Global::scenery->getID();
  cbox = new puaComboBox(30, BUTTON_BOX_HEIGHT + DLG_DEF_SPACE,
                        190, BUTTON_BOX_HEIGHT + DLG_DEF_BUTTON_HEIGHT + DLG_DEF_SPACE,
                        locations, false);
  cbox->setLabelPlace(PUPLACE_TOP_LEFT);
  cbox->setLabel("Select location:");
  close();
  setSize(220,120);
  setCallback(CGUILocationCallback);
  
  switch (curLoc)
  {
    case Scenery::CAPE_COD:
      cbox->setCurrentItem(0);
      break;
    
    case Scenery::DAVIS:
    default:
      cbox->setCurrentItem(1);
      break;
  }
  
  // center the dialog on screen
  int wwidth, wheight;
  int current_width = getABox()->max[0] - getABox()->min[0];
  int current_height = getABox()->max[1] - getABox()->min[1];
  puGetWindowSize(&wwidth, &wheight);
  setPosition(wwidth/2 - current_width/2, wheight*2/3 - current_height);

  reveal();
}


/**
 * Destroy the dialog.
 */

CGUILocationDialog::~CGUILocationDialog()
{
}


std::string CGUILocationDialog::getLocation() const
{
  std::string loc = cbox->getStringValue();
  return loc;
};

/** \brief The dialog's callback.
 *
 *  Load the new location.
 */
void CGUILocationCallback(puObject *obj)
{
  int curLoc = Global::scenery->getID();
  int newLoc = -1;
  std::string filename;

  if (obj->getIntegerValue() == CRRC_DIALOG_OK)
  {
    std::string loc = ((CGUILocationDialog*)obj)->getLocation();
    if (loc == "Davis")
    {
      newLoc = Scenery::DAVIS;
      filename = "scenery/davis-orig.xml";
    }
    else if (loc == "Cape Cod")
    {
      newLoc = Scenery::CAPE_COD;
      filename = "scenery/cape_cod-orig.xml";
    }

    if ((curLoc != newLoc) && (newLoc != -1))
    {
      cfg->setLocation(filename.c_str(), cfgfile);
      delete Global::scenery;
      clear_wind_field();
      Global::scenery = loadScenery(T_Config::getDataPath(filename).c_str());
      cfg->read(cfgfile);
      setWindowTitleString();
      *player_pos = Global::scenery->getPlayerPosition();
      initialize_wind_field();
      Global::Simulation->reset();
    }
  }
  puDeleteObject(obj);
  Global::gui->hide();
}
