// crrc_planesel.h - Airplane selection dialog

#ifndef CRRC_PLANESEL_H
#define CRRC_PLANESEL_H

#include <plib/pu.h>
#include <plib/puAux.h>
#include <vector>

#include "crrc_dialog.h"
#include "puaFileBox.h"
#include "puaScrListBox.h"
#include "puaGLPreview.h"
#include "../SimpleXMLTransfer.h"


class CGUIPlaneSelectDialog;

/** \brief The airplane selection dialog.
 *
 *  This dialog provides a scrollable list with all
 *  installed aircraft.
 */
class CGUIPlaneSelectDialog : public CRRCDialog
{
  public:
    CGUIPlaneSelectDialog();
    ~CGUIPlaneSelectDialog();
  
    /// Update the dialog depending on the currently selected file
    void  updateFileInfo();
    
    /// Assign a list of planes in the current category to planeList
    void  updatePlaneList();
  
    /// Update the OpenGL preview based on a UI change
    void updatePreview();

    /// Save selected file and options to config file
    bool  saveSelection() const;
  
    /// get value of the "load launch default" checkbox
    int getLoadLaunchDefault() const {return check_usedefault->getIntegerValue();};
  
  private:
    puaComboBox    *cat;
    char          **catList;
    int           catListSize;

    puaScrListBox *planes;
    char          **planeList;              ///< names of the planes
    int           planeListSize;
    std::vector<std::string> plane_paths;   ///< full paths of model files, same order as planeList

    puaGLPreview  *preview;                 ///< GL preview of the selected plane

    puaComboBox    *gbox;
    char**        optsGraphics;
    int           nOptsGraphics;
    SimpleXMLTransfer* optsGrpGraphics;

    puaComboBox    *cbox;
    char**        optsConfig;
    int           nOptsConfig;
    SimpleXMLTransfer* optsGrpConfig;
  
    puText        *location_label;
    std::string   location_label_string;
  
    puaLargeInput  *description;
    std::string   description_string;
    
    puButton      *check_usedefault;
  
    /// Fill the mfiles vector with the paths of all model files
    void  createMFileList(std::vector<std::string>& filelist);
    
    /// Assign a list of categories to the catList
    void  updateCategories();
    
    /// Get all planes of one category
    void  getPlanesByCategory(std::vector<std::string>& planes, std::string category);
    
    /// Get the name of a model
    std::string getModelName(std::string path);
    
    /// Clean up config boxes
    void cleanUpConfigAndGraphics();
    
};

#endif // CRRC_PLANESEL_H
