// puaFileBox.h - A list of file names.

#ifndef PUAFILEBOX_H
#define PUAFILEBOX_H

#include "puaScrListBox.h"
#include <vector>
#include <string>

/** \brief A scrollable list box filled with file names.
 *
 *  This class represents a scrollable PUI list box which
 *  is filled with file names. By specifying a file extension
 *  and a directory it's easy to automatically list a
 *  specific group of files.
 */
class puaFileBox : public puaScrListBox
{
  public:
    puaFileBox( int x, int y, int w, int h,
                const char *path, const char *extension,
                bool case_insensitive = false);
    virtual ~puaFileBox();
  
    virtual int addItems( const char *path, 
                          const char *extension,
                          bool case_insensitive = false);
    virtual const char * getPath(int index = -1);

  private:
    bool checkExtension(std::string fname,
                        std::string ext,
                        bool case_insensitive = false);
    std::vector<std::string *> files;
    std::vector<std::string *> paths;
    char          **entries;
};



#endif  // PUAFILEBOX_H
