// puaScrListBox.h - a scrolling PUI list box.
// Filename in FG sources: puList.hxx

#ifndef __PUASCRLISTBOX_H
#define __PUASCRLISTBOX_H

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <plib/pu.h>


/** \brief A scrolling list for PUI.
 * 
 *  This widget consists of a puListBox, a slider and two
 *  arrow buttons. This makes the ListBox scrollable,
 *  very handy if the box is too small to show all items
 *  at once.
 *
 *  (Original code taken from FlightGear 0.9.6 sources,
 *   modified by Jan Reucker)
 */
class puaScrListBox : public puGroup
{
  public:
    puaScrListBox (int x, int y, int w, int h);
    puaScrListBox (int x, int y, int w, int h, char ** contents);
    virtual ~puaScrListBox ();

    virtual void newList (char ** contents);

    virtual char * getStringValue ();
    virtual int    getIntegerValue ();
    virtual void   getValue (char **ps);
    virtual void   getValue (int  *i);
  
    virtual void   setValue (int i);

    virtual void   setStyle (int style);

    int  getNumItems ( void ) const ;
    int  getTopItem  ( void ) const ;
    void setTopItem  ( int item_index );


    /**
     *  Get the widget's current style.
     *  \return current style
     */
    virtual int    getStyle (void) const
    {
      return _style;
    };

  protected:
    virtual void init (int w, int h);

  private:
    char ** _contents;
    //~ puFrame * _frame;
    puListBox * _list_box;
    puSlider * _slider;
    puArrowButton * _up_arrow;
    puArrowButton * _down_arrow;
    int _style;
};

#endif // __PUASCRLISTBOX_H
