#ifndef UTIL_H
#define UTIL_H

#include "../SimpleXMLTransfer.h"

class T_GUI_Util
{
  public:
   
   /**
    * Loads a list of names from <code>grp</code> into an array,
    * memory is allocated as needed.
    * If fDefault then there always is the first entry 'default'. 
    * The last item in the list is a (char*)0.
    * nSize returns the number of entries (excluding (char*)0).
    */
   static char** loadnames(SimpleXMLTransfer* grp,
                           int&               nSize,
                           bool               fDefault = true);

   /**
    * Creates an array of names from the vector of strings,
    * memory is allocated as needed. The last item in the
    * list is a NULL-Pointer.
    * \param list list of names as vector of std::strings
    * \param nSize returns the number of entries (excluding NULL entry)
    * \return pointer to char[] containing the names
    */
   static char** loadnames( std::vector<std::string>  list,
                            int&                      nSize);

   /**
    * Frees memory allocated by the above methods.
    */
   static void   freenames(char**& ptr,
                           int     nSize);
  
   /// Compare a file extension to a given string
   static bool   checkExtension(std::string fname,
                                std::string ext,
                                bool case_insensitive = false);

    /** \brief Remove excessive whitespace from a string
     *
     *  This method removes any excessive whitespace from a string:
     *
     *  - leading whitespace characters
     *  - trailing whitespace characters
     *  - multiple successive whitespace characters
     *
     *  In addition, all remaining whitespace characters will be
     *  converted to plain spaces (' ').
     *
     *  \param  str The string to be trimmed.
     *  \return     String without excessive whitespace
     */
    static std::string trimWhitespace(std::string str);

    /** \brief Insert line-breaks into a string
     *
     *  This method inserts newline characters ('\n') into a string
     *  to create lines of at most line_length characters.
     *  If one single word on a line exceeds the maximum line_length,
     *  the newline character will be placed right behind this word.
     *  In this case the line will have more that line_length characters.
     *
     *  \param  str         input string
     *  \param  line_length desired line length
     *  \return string broken into lines of at most line_length characters
     */
    static std::string breakLines(std::string str, unsigned int line_length);
};

#endif
