#include <stdio.h>
//#include <stdlib.h>
//#include <string.h>
//#include <sys/time.h>

#include "T_TX_InterfaceSerial.h"
#include "lib_conversions.h"
#include "global.h"
#include <sstream>
//#include "chardevice/chardevicebase.h"

// Remaining problems:
//   - crash when non-existent port is opened
//   - connect USB, choose USB, save, quit, disconnect USB, start ==> crash?

// TODO Kalibrierproblem

// TODO 110 Baud eingestellt nach Fehler

#define DEFAULT_PORT_NAME serialDevs[0]

#if DEBUG_TX_INTERFACE > 0
#define DEBUG(...) printf (__VA_ARGS__)
#else
#define DEBUG()
#endif

const int T_TX_InterfaceSerial::nNumBaudrates = 11;
char *T_TX_InterfaceSerial::baudRates[] = 
                            {    "110",  "300",   "600",  "1200",  "2400",
                                 "4800", "9600", "19200", "38400", "57600",
                                 "115200",   NULL };
const int T_TX_InterfaceSerial::anBaudRates[] = 
                            {      110,    300,     600,    1200,    2400,
                                  4800,   9600,   19200,   38400,   57600,
                                115200,     -1 };
  

// ******************
// ** Construction **
// ******************

T_TX_InterfaceSerial::T_TX_InterfaceSerial ()/*{{{*/
	:charDevice (NULL), portName (""), baudRate (0)
{
	DEBUG ("T_TX_InterfaceSerial::T_TX_InterfaceSerial ()\n");

	mixer=new T_TX_Mixer (this);
	map=new T_AxisMapper (this);
	calib=new T_Calibration (this);

#if DEBUG_TX_INTERFACE>0
	dbgbufferidx=0;
#endif
}
/*}}}*/

T_TX_InterfaceSerial::~T_TX_InterfaceSerial ()/*{{{*/
{
	DEBUG("T_TX_InterfaceSerial::~T_TX_InterfaceSerial()\n");

	delete mixer;
	delete map;
	delete calib;
	delete charDevice;
}
/*}}}*/

// *******************
// ** Configuration **
// *******************

int T_TX_InterfaceSerial::init (SimpleXMLTransfer *config)/*{{{*/
{
	DEBUG ("int T_TX_InterfaceSerial::init (SimpleXMLTransfer *config)\n");

	try
	{
		// Initialize the port settings
		SimpleXMLTransfer *port=config->getChild (getXmlChildName ()+".port", true);
		portName=port->getString ("name", DEFAULT_PORT_NAME);
		baudRate=port->getInt ("baudrate", getDefaultBaudRate ());
	  
		// Initialize the subsystems
		if (mixer->init (config, getXmlChildName ())!=0)
			return -1;
		
		calib->init (config, getXmlChildName ());
		map->init (config, getXmlChildName ());

		// Try-Methode?
		ostringstream options;
		options << portName << "," << baudRate;
		string optionString=options.str ();
		cout << "Opening the serial port with option string " << optionString << endl;
		charDevice=new SerialCharDevice (optionString.c_str (), false);
	}
	catch (CharDevice::ConfigureDeviceException e)
	{
		setErrMsg ("The device could not be configured.");
		cout << "Serial interface initialization: " << getErrMsg () << endl;
		return 1;
	}

	cout << "Serial interface initialization: OK" << endl;
	return 0;
}
/*}}}*/

void T_TX_InterfaceSerial::putBackIntoCfg (SimpleXMLTransfer *config)/*{{{*/
{
	DEBUG("int T_TX_InterfaceSerial::putBackIntoCfg (SimpleXMLTransfer *config)\n");

	// Store the port settings
	SimpleXMLTransfer *port=config->getChild (getXmlChildName ()+".port", true);
	port->setAttributeOverwrite ("name", portName);
	port->setAttributeOverwrite ("baudrate", baudRate);

	// Store the subsystems
	if (mixer) mixer->putBackIntoCfg (config);
	if (map)   map  ->putBackIntoCfg (config);
	if (calib) calib->putBackIntoCfg (config);
}
/*}}}*/

int T_TX_InterfaceSerial::getDeviceNumber (const char* list[])/*{{{*/
{
	for (int i=0; list[i]!=NULL; ++i)
		if (portName.compare (list[i])==0)
			return i;

	return -1;
}
/*}}}*/

void T_TX_InterfaceSerial::setDeviceSpeed (int speed)/*{{{*/
{
  if ((speed >= 0) && (speed < nNumBaudrates))
  {
    baudRate = anBaudRates[speed];
  }
  else
  {
    baudRate = getDefaultBaudRate();
  }
}
/*}}}*/

int T_TX_InterfaceSerial::baudRateToSpeed (int br)/*{{{*/
{
  int nSpeed;

	switch (br)
	{
		case 110:
      nSpeed = 0;
      break;
		
    case 300:
      nSpeed = 1;
      break;
    
		case 600:
      nSpeed = 2;
      break;
		
    case 1200:
      nSpeed = 3;
      break;
		
    case 2400:
      nSpeed = 4;
      break;
		
    case 4800:
      nSpeed = 5;
      break;
    
		case 9600:
      nSpeed = 6;
      break;
    
		case 19200:
      nSpeed = 7;
      break;
    
		case 38400: 
      nSpeed = 8;
      break;
    
		case 57600: 
      nSpeed = 9;
      break;
    
		case 115200:
      nSpeed = 10;
      break;
    
    default:
      nSpeed = -1;
      break;
	}

	return nSpeed;
}
/*}}}*/

int T_TX_InterfaceSerial::getDeviceSpeed ()/*{{{*/
{
	int speed = baudRateToSpeed (baudRate);
	if (speed < 0) 
    speed = baudRateToSpeed (getDefaultBaudRate ());
	return speed;
}
/*}}}*/

void T_TX_InterfaceSerial::setDeviceName (std::string devname)/*{{{*/
{
	portName=devname;
}
/*}}}*/

void T_TX_InterfaceSerial::setDtr (bool dtr)/*{{{*/
{
	if (dtr)
		charDevice->write_output_pins (SerialCharDevice::WriteOutputPinUnchanged, SerialCharDevice::WriteOutputPinHigh); // RTS, DTR
	else
		charDevice->write_output_pins (SerialCharDevice::WriteOutputPinUnchanged, SerialCharDevice::WriteOutputPinLow); // RTS, DTR
}
/*}}}*/

void T_TX_InterfaceSerial::setRts (bool rts)/*{{{*/
{
	if (rts)
		charDevice->write_output_pins (SerialCharDevice::WriteOutputPinHigh, SerialCharDevice::WriteOutputPinUnchanged); // RTS, DTR
	else
		charDevice->write_output_pins (SerialCharDevice::WriteOutputPinLow, SerialCharDevice::WriteOutputPinUnchanged); // RTS, DTR
}
/*}}}*/


// *****************
// ** Data access **
// *****************

void T_TX_InterfaceSerial::getInputData (TSimInputs *inputs)/*{{{*/
{
	// Read serial data and update rawData[]
	readSerialData ();

	// If a function is used (it has a channel), set the corresponding input
	if (map->func[ELEVATOR]>=0)
    inputs->elevator=mixer->mix_elevator (calibrateValue (ELEVATOR));
	if (map->func[AILERON]>=0)
    inputs->aileron=mixer->mix_aileron (calibrateValue (AILERON));
	if (map->func[RUDDER]>=0)
    inputs->rudder=mixer->mix_rudder (calibrateValue (RUDDER));
	if (map->func[THROTTLE]>=0)
    inputs->throttle=mixer->mix_throttle (calibrateValue (THROTTLE)+0.5);

	//DEBUG ("\rElevator: %+.2f, Aileron: %+.2f, Rudder: %+.2f, Throttle: %+.2f   ", inputs->elevator, inputs->aileron, inputs->rudder, inputs->throttle);
}
/*}}}*/

void T_TX_InterfaceSerial::getRawData (float *dest)/*{{{*/
{
	// Read serial data and update rawData[]
	readSerialData ();

	int numAxes=getNumAxes ();
	for (int i=0; i<numAxes; ++i)
		*(dest+i)=rawData[i];
}
/*}}}*/

double T_TX_InterfaceSerial::calibrateValue (int function)/*{{{*/
{
	int axis=map->func[function];
	return calib->calibrate (axis, rawData[axis]) * map->inv[function];
}
/*}}}*/


// *********
// ** I/O **
// *********

/**
 * Reads one byte from the serial interface. Only this method should be used
 * for reading serial data.
 * \return -1 if no more bytes available, the byte else
 */
int T_TX_InterfaceSerial::getSerialByte ()/*{{{*/
{
	unsigned char buffer;

	if (charDevice->read (&buffer, 1)<1)
		return -1;

  #if DEBUG_TX_INTERFACE > 0
  debugHandler(buffer);
  #endif

	return buffer;
}
/*}}}*/

void T_TX_InterfaceSerial::readSerialData ()/*{{{*/
{
	while (true)
	{
		int data=getSerialByte ();
		if (data<0) return;

		processDataByte (data);
	}
}
///*}}}*/

void T_TX_InterfaceSerial::setRawData (int num, float data)/*{{{*/
{
//	DEBUG ("setting %d to %f\n", num, data);

	if (num>=0 && num<TX_MAXAXIS)
		rawData[num]=data;
}
/*}}}*/


#if DEBUG_TX_INTERFACE > 0
/**
 *  Generate some interesting debugging output.
 *
 *  Dump the byte stream received from the serial port to stdout.
 *  If DEBUG_TX_INTERFACE is set to 1, DBGBUFFERSIZE bytes are
 *  recorded once at startup and then dumped to stdout. If
 *  DEBUG_TX_INTERFACE is set to 2, the byte stream is continously
 *  dumped to stdout in chunks of DBGBUFFERSIZE bytes.
 *
 *  \param ucByte   the byte to be logged
 */
void T_TX_InterfaceSerial::debugHandler(unsigned char ucByte)
{
  if (dbgbufferidx < DBGBUFFERSIZE)
  {
    dbgbuffer[dbgbufferidx++] = ucByte;
  }
  else if (dbgbufferidx == DBGBUFFERSIZE)
  {
    #if DEBUG_TX_INTERFACE == 1
    dbgbufferidx++;
    #elif DEBUG_TX_INTERFACE == 2
    dbgbufferidx = 0;
    #endif
    for (int i = 0; i < DBGBUFFERSIZE; i++)
    {
      if (i % 32 == 0)
      {
        printf("\n");
      }
      printf("%02X ", dbgbuffer[i]);
    }
    printf("\n");
  }
}
#endif // DEBUG_TX_INTERFACE > 0
