/** \file crrc_animation.h
 *
 *  Classes for animated model parts.
 */

#ifndef CRRC_ANIMATION_H
#define CRRC_ANIMATION_H

#include <vector>

#include <plib/ssg.h>
#include "SimpleXMLTransfer.h"
#include "crrc_inputs.h"


// --- non-class-functions -----------

void initAnimations(SimpleXMLTransfer *model_file, ssgEntity* model, TSimInputs *fInput);


/** A base class for all animations
 *
 *  This is the ABC for all animations, providing a
 *  unified interface to the scene rendering engine.
 */
class CRRCAnimation : public ssgBase
{
  public:
    /// Constructor
    CRRCAnimation(ssgBranch *branch);
  
    /// Destructor
    virtual ~CRRCAnimation();
  
    /// Initialize the animation
    virtual void init();
  
    /// Update the animation before rendering
    virtual int update() = 0;
  
    /// get-method for the animation branch
    inline ssgBranch* getBranch()  {return anim_branch;};
  
  protected:
    ssgBranch *anim_branch;
};


/** Rotate a control surface
 *
 *  This animation rotates a control surface according to
 *  the control input.
 */
class CRRCControlSurfaceAnimation : public CRRCAnimation
{
  public:
    /// Constructor
    CRRCControlSurfaceAnimation(SimpleXMLTransfer *xml, TSimInputs *inputs);
  
    /// Destructor
    ~CRRCControlSurfaceAnimation();
  
    /// Initialize the animation
    void init();
  
    /// Update the animation before rendering
    int update();
  
  private:
    std::vector<float*> datasource;
    std::vector<float>  source_gain;
    float fallback_data;
    sgVec3 hinge_1;
    sgVec3 hinge_2;
    sgVec3 axis;
    sgMat4 move_to_origin;
    sgMat4 move_back;
    float  max_angle;
    float  abs_max_angle;
};


#endif // CRRC_ANIMATION_H
