/***************************************************************************

 Title:         crrc_checkopts.c

 DESIGNED:      Kees Lemmens, July 2004
 MODIFIED:      KL, August 2004 , options to bind all joystick axes and buttons
 MODIFIED:      TT, December, 2006 , -g, -s, -u, mnav interface
 CODED:         KL
 MAINTAINER:    KL
 LAST MODIFIED: July 2004
----------------------------------------------------------------------------*/

#include <unistd.h>

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

extern char   *optarg;
extern int    optind;

#include "global.h"
#include "crrc_main.h"
#include "crrc_main_cpp.h"

void crrc_usage(char *progname)
{
  fprintf(stderr,"\nUsage  : %s [options] [<plane>]\n",progname);
  fprintf(stderr,"\nNote that these options override default settings in crrcsim.xml !\n");
  fprintf(stderr,  "Options:\n");
  fprintf(stderr,  "         -h             : display this message\n");
  fprintf(stderr,  "         -l <string>    : location/scenery file with path (e.g. scenery/davis-orig.xml)\n");
  fprintf(stderr,  "                          Use this option before others, otherwise they might be overwritten.\n");
  fprintf(stderr,  "         -b <nr:string> : joystick buttonnr function: RESUME|RESET|PAUSE|ZOOMIN|ZOOMOUT|INCTHROTTLE|DECTHROTTLE\n");
  fprintf(stderr,  "         -c <value>     : color_depth in bits per pixel\n");
  fprintf(stderr,  "         -d <value>     : wind direction in deg (0-360)\n");
  fprintf(stderr,  "         -f             : use fullscreen\n");
  fprintf(stderr,  "         -g <string>    : specify config file\n");
  fprintf(stderr,  "         -i <string>    : input method : KEYBOARD/MOUSE/JOYSTICK/RCTRAN/SERIAL2/PARALLEL/AUDIO/MNAV\n");
  fprintf(stderr,  "         -j <nr:string> : joystick axisnr function: AILERON|RUDDER|ELEVATOR|THROTTLE\n");
  fprintf(stderr,  "         -m <string>    : mouse x motion : AILERON|RUDDER\n");
  fprintf(stderr,  "         -s <on/off>    : sound on/off\n");
  fprintf(stderr,  "         -u <on/off>    : user interface on/off\n");
  fprintf(stderr,  "         -w <value>     : wind velocity in ft/sec\n");
  fprintf(stderr,  "         -x <value>     : x_resolution in pixels\n");
  fprintf(stderr,  "         -y <value>     : y_resolution in pixels\n");
  fprintf(stderr,  "         -v             : Show input values\n");
  fprintf(stderr,  "         -v             : Show current field of view\n");
  fprintf(stderr,  "         -v             : Show frames per second\n");
  fprintf(stderr, "\n");
}

/**
 * returns 
 *   0  everything OK, let's start
 *  -1  error
 *   1  test of transmitter interface requested
 */
int crrc_checkopts (int                argc,
                    char*              argv[], 
                    SimpleXMLTransfer* cfgfile,
                    T_Config*          cfg)  /* check and set options flags */
{
  int c;
  int axisnr, buttonnr;
  int opt_err = 0;
  int new_res_x = 0;
  int new_res_y = 0;

  while ((c = getopt(argc, argv, "b:c:d:fg:hi:j:l:m:s:u:vw:x:y:")) != EOF)
  {
    switch (c)
    {
      case 'b':
        if (! isdigit(optarg[0]) || optarg[1] != ':')
        {
          opt_err = 1;
          break;
        }
        buttonnr=atoi(optarg);
        if(buttonnr > MAXJOYBUTTON)
        {
          fprintf(stderr, "illegal button nr: %d\n", buttonnr);
          crrc_exit(CRRC_EXIT_FAILURE,"");
        }
        optarg += 2;
        if (strcasecmp(optarg,"RESUME")==0)
          inputDev.joystick_bind_b[buttonnr] = RESUME;
        else if (strcasecmp(optarg,"RESET")==0)
          inputDev.joystick_bind_b[buttonnr] = RESET;
        else if (strcasecmp(optarg,"PAUSE")==0)
          inputDev.joystick_bind_b[buttonnr] = PAUSE;
        else if (strcasecmp(optarg,"DECTHROTTLE")==0)
          inputDev.joystick_bind_b[buttonnr] = DECTHROTTLE;
        else if (strcasecmp(optarg,"INCTHROTTLE")==0)
          inputDev.joystick_bind_b[buttonnr] = INCTHROTTLE;
        else if (strcasecmp(optarg,"ZOOMIN")==0)
          inputDev.joystick_bind_b[buttonnr] = ZOOMIN;
        else if (strcasecmp(optarg,"ZOOMOUT")==0)
          inputDev.joystick_bind_b[buttonnr] = ZOOMOUT;
        break;
      case 'c':
        cfgfile->setAttributeOverwrite("video.color_depth", optarg);
        break;
      case 'd':
        cfg->wind->setDirection((float)atof(optarg), cfg);
        break;
      case 'f':
        cfgfile->setAttributeOverwrite("video.fullscreen.fUse", "1");
        break;
      case 'g':
        // handled in crrc_main.cpp before this is called
        break;
      case 'h':
        crrc_usage(argv[0]);
        crrc_exit(CRRC_EXIT_SUCCESS,"");
        break;
      case 'v':
        Global::nVerbosity++;
        break;
      case 'i':
        cfgfile->setAttributeOverwrite("inputMethod.method", optarg);
        break;
      case 'j':
        if (! isdigit(optarg[0]) || optarg[1] != ':')
        {
          opt_err = 1;
          break;
        }
        axisnr=atoi(optarg);
        if(axisnr > MAXJOYAXIS)
        {
          fprintf(stderr, "illegal axis nr: %d\n", axisnr);
          crrc_exit(CRRC_EXIT_FAILURE,"");
        }
        optarg += 2;
        if (strcasecmp(optarg,"AILERON")==0)
          inputDev.joystick_bind_x[axisnr] = AILERON;
        else if (strcasecmp(optarg,"RUDDER")==0)
          inputDev.joystick_bind_x[axisnr] = RUDDER;
        else if (strcasecmp(optarg,"ELEVATOR")==0)
          inputDev.joystick_bind_x[axisnr] = ELEVATOR;
        else if (strcasecmp(optarg,"THROTTLE")==0)
          inputDev.joystick_bind_x[axisnr] = THROTTLE;
        break;
      case 'l': /* airport location */
        cfg->setLocation(optarg, cfgfile);
        break;
      case 'm':
        if (strcasecmp(optarg,"AILERON")==0)
          inputDev.mouse_bind_x = AILERON;
        else if (strcasecmp(optarg,"RUDDER")==0)
          inputDev.mouse_bind_x = RUDDER;
        break;
      case 's':
        if      (strcasecmp(optarg,"ON")==0)
          cfgfile->setAttributeOverwrite("sound.enabled", "1");
        if      (strcasecmp(optarg,"OFF")==0)
          cfgfile->setAttributeOverwrite("sound.enabled", "0");
        break;
      case 'u':
        if      (strcasecmp(optarg,"ON")==0)
          cfgfile->setAttributeOverwrite("video.enabled", "1");
        if      (strcasecmp(optarg,"OFF")==0)
          cfgfile->setAttributeOverwrite("video.enabled", "0");
        break;
      case 'w':
        cfg->wind->setVelocity((float)atof(optarg));
        break;
      case 'x':
        new_res_x = atoi(optarg);
        break;
      case 'y':
        new_res_y = atoi(optarg);
        break;
      default:
        opt_err = 1;
    }
    
    if (opt_err)
    {
       crrc_usage(argv[0]);
       return(-1);
    }
  }
   
  // x and y resolution must be stored after all other flags have
  // been evaluated to make sure that we really know if the user
  // wants fullscreen or windowed mode.
  int use_fullscreen = cfgfile->getInt("video.fullscreen.fUse", 0);
  if (new_res_x > 0)
  {
    if (use_fullscreen)
    {
      cfgfile->setAttributeOverwrite("video.resolution.fullscreen.x", new_res_x);
    }
    else
    {
      cfgfile->setAttributeOverwrite("video.resolution.window.x", new_res_x);
    }
  }
  if (new_res_y > 0)
  {
    if (use_fullscreen)
    {
      cfgfile->setAttributeOverwrite("video.resolution.fullscreen.y", new_res_y);
    }
    else
    {
      cfgfile->setAttributeOverwrite("video.resolution.window.y", new_res_y);
    }
  }
   
  if (argc - optind != 0)
    cfgfile->setAttributeOverwrite("airplane.file", argv[optind]);
  
  return 0;
}

