/***************************************************************************
 
 Title:  crrc_keyboard.c
 
 DESIGNED: Kees Lemmens, July 2004
 CODED:    KL
 MODIFIED: TT, December, 2006 , ability to operate with no gui,
                 added 4 auxiliary 3-way switches
                 (triggered by F{1,2,3,4} and then {1,2,3})
 MAINTAINER: KL
 LAST MODIFIED: July 2004
----------------------------------------------------------------------------*/

#include "global.h"
#include "SimStateHandler.h"
#include "crrc_main.h"
#include "crrc_main_cpp.h"
#include "defines.h"
#include "zoom.h"
#include "GUI/crrc_gui_main.h"

#if DEBUG_THERMAL_SCRSHOT == 1
# include "windfield.h"
# include "fdm.h"
#endif

/*****************************************************************************/

void key_down(SDL_keysym *keysym)
{
  static int current_aux = -1;
  int received_begin_aux = 0;

  switch (keysym->unicode)
  {
    case 'q':
    case 'Q':
      if (Global::gui) 
        Global::gui->doQuitDialog();
      else
        Global::Simulation->quit();
      break;
  
    default:
      /* just to avoid compiler complains ;) */
      break;
  } 

  switch (keysym->sym)
  {
    case SDLK_r:
      Global::Simulation->reset();
      break;

    case SDLK_p:
      Global::Simulation->pause();
      LOG("Press <u> to resume.");
      break;

    case SDLK_u:
      Global::Simulation->resume();
      break;

    case SDLK_d:
      if (!(Global::gui && Global::gui->isVisible()))
      {
        if (Global::testmode.test_mode)
        {
          Global::Simulation->reset();
        }
        else
        {
          activate_test_mode();
        }
      }
      break;

    case KEY_ZOOM_OUT:
      if (inputDev.zoom_control == KEYBOARD)
      {
        zoom_out();
      }
      break;

    case SDLK_t:
      if (Global::training_mode == FALSE)
      {
        Global::training_mode = TRUE;
        LOG("Training mode is ON, thermals are visible.");
      }
      else
      {
        Global::training_mode = FALSE;
        LOG("Training mode is OFF.");
      }
      break;

    case KEY_ZOOM_IN:
      if (inputDev.zoom_control == KEYBOARD)
      {
        zoom_in();
      }
      break;

    case SDLK_KP5:
      TXInterface->centerControls();
      break;

    case SDLK_LEFT:
    case SDLK_KP4:
      TXInterface->move_rudder(0.05);
      break;

    case SDLK_RIGHT:
    case SDLK_KP6:
      TXInterface->move_rudder(-0.05);
      break;

    case SDLK_KP7:
      TXInterface->move_aileron(-0.05);
      break;

    case SDLK_KP9:
      TXInterface->move_aileron(0.05);
      break;

    case SDLK_UP:
    case SDLK_KP8:
      TXInterface->move_elevator(0.05);
      break;

    case SDLK_DOWN:
    case SDLK_KP2:
      TXInterface->move_elevator(-0.05);
      break;

    case KEY_THROTTLE_MORE:
      TXInterface->increase_throttle();
      break;

    case KEY_THROTTLE_LESS:
      TXInterface->decrease_throttle();
      break;   
    case SDLK_F1:
      current_aux = 1;
      received_begin_aux = 1;
      break;
    case SDLK_F2:
      current_aux = 2;
      received_begin_aux = 1;
      break;
    case SDLK_F3:
      current_aux = 3;
      received_begin_aux = 1;
      break;
    case SDLK_F4:
      current_aux = 4;
      received_begin_aux = 1;
      break;
    case SDLK_1:
      if (current_aux >= 0)
        set_aux(current_aux, 1);
      break;
    case SDLK_2:
      if (current_aux >= 0)
        set_aux(current_aux, 2);
      break;
    case SDLK_3:
      if (current_aux >= 0)
        set_aux(current_aux, 3);
      break;
    /*case SDLK_q:
      if (gui)
        gui->doQuitDialog();
      break;*/
#if DEBUG_THERMAL_SCRSHOT == 1
    case SDLK_s:
      windfield_thermalScreenshot(Global::fdm->airplane->getPos());
      break;
#endif
    default:
      /* just to avoid compiler complaints ;) */
      break;
  }
  
  if (!received_begin_aux)
    current_aux = -1;
}

