#include "fdm.h"

#include <iostream>

#include "CVector.h"

#include "fdm_config.h"
#if (MOD_FDM_USE_002 != 0)
# include "fdm_002/fdm_002.h"
#endif
#if (MOD_FDM_USE_LARCSIM != 0)
# include "fdm_larcsim/fdm_larcsim.h"
#endif
#if (MOD_FDM_USE_DISPLAYMODE != 0)
# include "fdm_displaymode/fdm_displaymode.h"
#endif
#if (MOD_FDM_USE_TESTMODE != 0)
# include "fdm_testmode/fdm_testmode.h"
#endif
#include "xmlmodelfile.h"

CRRC_FDM::CRRC_FDM()
 : launch_presets(NULL)
{
  airplane = (CRRC_AirplaneSim*)0;
}

CRRC_FDM::~CRRC_FDM()
{
  if (launch_presets != NULL)
    delete launch_presets;
  if (airplane != (CRRC_AirplaneSim*)0)
    delete airplane;
}

void CRRC_FDM::update(TSimInputs* inputs,
                      double      dt,
                      int         multiloop)
{
  airplane->update(inputs, dt, multiloop);
}

void CRRC_FDM::initAirplaneState(double dRelVel,
                                 double dTheta,
                                 double dPsi,
                                 double X,
                                 double Y,
                                 double Z,
                                 double R_X,
                                 double R_Y,
                                 double R_Z)
{
  airplane->initAirplaneState(dRelVel,
                              dTheta,
                              dPsi,
                              X,
                              Y,
                              Z,
                              R_X,
                              R_Y,
                              R_Z);
}

void CRRC_FDM::Clean()
{
  if (airplane != (CRRC_AirplaneSim*)0)
  {
    delete airplane;
  }
  if (launch_presets != NULL)
  {
    delete launch_presets;
  }
  airplane = 0;
  launch_presets = 0;
}

void CRRC_FDM::loadAirplaneTestmode(double dNorth, double dEast, double dDown)
{
  Clean();
  
#if (MOD_FDM_USE_TESTMODE != 0)
  airplane = new CRRC_AirplaneSim_TestMode(CRRCMath::Vector3(dNorth, dEast, dDown));
#endif
}

void CRRC_FDM::loadAirplane(const char* filename, 
                            FDMEnviroment* myEnv,
                            SimpleXMLTransfer* cfg)
{
  Clean();
  
  // todo: find out the type of model to be used by extension or by some value in 'cfg'?

  // Try all available FDMs until one of them doesn't throw an exception:

#if (MOD_FDM_USE_DISPLAYMODE != 0)
  if (airplane == 0 && CRRC_AirplaneSim_DisplayMode::UseMe(cfg))
  {
    try
    {
      airplane = new CRRC_AirplaneSim_DisplayMode(filename, cfg);
    }
    catch (XMLException e)
    {
      std::cerr << "Not CRRC_AirplaneSim_DisplayMode: " << filename << "\n";
      airplane = 0;
    }
  }  
#endif
  
#if (MOD_FDM_USE_LARCSIM != 0)
  if (airplane == 0)
  {
    try
    {
      airplane = new CRRC_AirplaneSim_Larcsim(filename, myEnv);
    }
    catch (XMLException e)
    {
      std::cerr << "Not CRRC_AirplaneSim_Larcsim: " << filename << "\n";
      airplane = 0;
    }
  }
#endif
}

void CRRC_FDM::loadAirplane(SimpleXMLTransfer* xml, 
                            FDMEnviroment* myEnv,
                            SimpleXMLTransfer* cfg)
{
  Clean();
    
  // todo: find out the type of model to be used by some value in 'xml' or 'cfg'?

  // Try all available FDMs until one of them doesn't throw an exception:

#if (MOD_FDM_USE_DISPLAYMODE != 0)
  if (airplane == 0 && CRRC_AirplaneSim_DisplayMode::UseMe(cfg))
  {
    try
    {
      airplane = new CRRC_AirplaneSim_DisplayMode(xml, cfg);
    }
    catch (XMLException e)
    {
      std::cerr << "Not CRRC_AirplaneSim_DisplayMode: " << e.what() << "\n";
      airplane = 0;
    }
  }
#endif
  
#if (MOD_FDM_USE_LARCSIM != 0)
  if (airplane == 0)
  {
    try
    {
      airplane = new CRRC_AirplaneSim_Larcsim(xml, myEnv);
    }
    catch (XMLException e)
    {
      std::cerr << "Not CRRC_AirplaneSim_Larcsim: " << e.what() << "\n";
      airplane = 0;
    }
  }
#endif  
  // try to load launch presets from the airplane file
  launch_presets = XMLModelFile::getLaunchPresets(xml);
}

CRRC_AirplaneSim::CRRC_AirplaneSim(const char* logfilename, FDMEnviroment* myEnv)
{
  env = myEnv;
#if FDM_LOG != 0
  std::cout << "Opening fdm logfile\n";
  logfile.open(logfilename);
  nLogCnt = 0;
  nStep   = 0;
#endif
}

CRRC_AirplaneSim::~CRRC_AirplaneSim()
{
#if FDM_LOG != 0
  std::cout << "Closing fdm logfile\n";
  logfile.close();
#endif
}
