#ifndef ENVIROMENT_H
# define ENVIROMENT_H

/**
 * This is the interface used by the (various) FDMs to get information from the outside:
 *   - scenery for collision detection
 *   - windfield data
 * An instance of an implementation of this is passed to the FDM upon creation. This seems
 * quite expensive, but helps to separate this module (FDM) quite well.
 * 
 * @author Jens Wilhelm Wulf
 */
class FDMEnviroment
{
public:
  
  virtual ~FDMEnviroment() {};
  
  /**
   *  Get the height at a distinct point.
   *  \param x x coordinate (positive north)
   *  \param y y coordinate (positive east)
   *  \return terrain height at this point in ft
   */
  virtual float GetSceneryHeight(float x_north, float y_east) = 0;
  
  /**
   * Calculate the wind velocities in all three axes in the given position.
   * Returns 1 if this position is outside of the grid.
   * X/Y/Z -- north/east/down
   */
  virtual int CalculateWind(double  X_cg,      double  Y_cg,     double  Z_cg,
                            double& Vel_north, double& Vel_east, double& Vel_down) = 0;
};

#endif
