// -*- mode: c; mode: fold -*-
//
#ifndef FDM_TESTMODE_H
# define FDM_TESTMODE_H

# include <stdexcept>
# include "../ls_types.h"
# include "../fdm.h"
# include "../math/vector3.h"

/**
 *
 * This is not a real flight dynamics model. It only shows simple reactions 
 * to inputs in order to check whether input channels are connected as intended.
 *
 * @author Jens Wilhelm Wulf
 */
class CRRC_AirplaneSim_TestMode : public CRRC_AirplaneSim /*{{{*/
{
   friend class CRRC_FDM;
   
  public:

   virtual bool   isStalling() { return(0); };
   virtual CRRCMath::Vector3 getPos();   
   virtual double getPhi();
   virtual double getTheta();
   virtual double getPsi();

   /**
    * Used for sound calculation. It returns the prop's number of revolutions
    * per second [1/s].
    * jwtodo: simply return something related to throttle input
    */
   double getPropFreq() { return(0); };

   /**
    * Returns velocity relative to airmass [ft/s].
    */
   virtual double getVRelAirmass() { return(0); };
   
   /**
    * the longest distance from any of the aircrafts points to the CG
    */
   virtual double getAircraftSize() { return(1); };

   /**
    * computed velocity for trimmed flight in dead air
    */
   virtual double getTrimmedFlightVelocity() {return(1); };

   /**
    * returns Z coordinate of lowest point
    */
   virtual double getZLow() { return(1); };

   /**
    * Wingspan of the aircraft in feet
    */
   virtual double getWingspan() { return(1.0); };
   
  private:
   void update(TSimInputs* inputs,
               double      dt,
               int         multiloop);

   virtual void initAirplaneState(double dRelVel,
                                  double dTheta,
                                  double dPsi,
                                  double X,
                                  double Y,
                                  double Z,
                                  double R_X,
                                  double R_Y,
                                  double R_Z);
   
   /**
    * Airplane position is at ipos
    */
   CRRC_AirplaneSim_TestMode(CRRCMath::Vector3 ipos);

   virtual ~CRRC_AirplaneSim_TestMode();

  private:
   CRRCMath::Vector3 basepos;
   CRRCMath::Vector3 pos;
   CRRCMath::Vector3 angle;
   double  dPsi0;
};
/*}}}*/

#endif
