/** \file global.h
 *
 *  Global variables used in CRRCsim.
 */

#ifndef GLOBAL_H
#define GLOBAL_H

//~ #include "GUI/crrc_gui_main.h"
//~ #include "crrc_scenery.h"
//~ #include "crrc_loadair.h"
//~ #include "crrc_soundserver.h"

#include <string>
#include "crrc_inputs.h"
#include "glconsole.h"      // needed to make LOG() work without add. headers
// There's no need to pull in the full headers here.
// Just declare the classes and leave the responsibility
// to the files that really need the single variables.
class SimStateHandler;
class CGUIMain;
class CRRCAirplane;
class CRRCAudioServer;
class CRRC_FDM;
class Scenery;
class T_GameHandler;
//class TSimInputs;


/**
 * Contains data related to test mode.
 * 
 * @author Jens Wilhelm Wulf
 */
struct TestModeData
{
  /**
   *  test TXInterface
   */
  int   test_mode;
  
  /**
   * The real flAutozoom is changed while in testmode
   */
  float flAutozoom;
  
  /**
   * This pointer stores the pointer to the real fdm as long as the
   * testmode fdm is in use.
   */
  CRRC_FDM* fdm_backup;
};

class Global
{
  public:
    static SimStateHandler* Simulation;     ///< The simulation's main state machine.
    static int              training_mode;  ///< Draw thermals in the sky?
    static int              nVerbosity;     ///< How much info in the HUD?
    static Scenery*         scenery;        ///< The scenery.
    static CGUIMain*        gui;            ///< The GUI.
    static CRRCAirplane*    airplane;       ///< The airplane model.
    static CRRC_FDM*        fdm;            ///< The fdm which is in use.
    static CRRCAudioServer* soundserver;    ///< The sound server.
    static T_GameHandler*   gameHandler;    ///< The active game mode.
    static TSimInputs       inputs;         ///< Control input values.
    static float            dt;             ///< time interval of integration of EOMs
    static std::string      verboseString;  ///< Informational line of text
    static TestModeData     testmode;       ///< Test mode data structure
    static GlConsole*       console;        ///< The console
    static int              nFPS;           ///< average video update rate (FPS)
};



/**
 * A list of names for serial devices. Depends on OS.
 */
extern const char* serialDevs[];

/** This macro logs a line of text to the console */
#define LOG(_x)     {if (Global::console != NULL) Global::console->print(_x);}

#endif //GLOBAL_H
