/** \file gloverlay.h
 *
 *  This file defines the interface of the GlOverlay class
 *  which a base class for 2D overlays in a PLIB rendering
 *  environment.
 *
 *  \author Jan Reucker (slowhand_47@gmx.de)
 */

#ifndef _GLOVERLAY_H
#define _GLOVERLAY_H

#include <list>

#define DEBUG_GLOVERLAY 1

/**
 *  Base class for 2D overlays
 *  
 *  This abstract base class provides basic mechanisms common to all
 *  2D overlays.
 */
class GlOverlay
{
  public:
    /** 
     *  constructor
     */
    GlOverlay();
  
    /**
     *  destructor
     */
    virtual ~GlOverlay();
    
    /**
     *  Draw the overlay
     *
     *  Derived classes must implement this method.
     */
    virtual void draw() const = 0;
  
    /**
     *  Draw all overlays
     */
    static void renderAllInstances(int window_xsize, int window_ysize);

    /**
     *  Switch to the 2D rendering state
     */
    static void setupRenderingState(int window_xsize, int window_ysize);
  
    /**
     *  Switch back to the original rendering state
     */
    static void restoreRenderingState();
  
  protected:
    static int window_size_x;
    static int window_size_y;

  private:
    static std::list<GlOverlay*> GlOverlayInstances;
  
    void registerInstance(GlOverlay *ovl);
    void unregisterInstance(GlOverlay *ovl);
};

#endif    // _GLOVERLAY_H
