#ifndef TX_INTERFACE_Parallel_H
#define TX_INTERFACE_Parallel_H

#include "../tx_interfacePPM.h"
#include "../SimpleXMLTransfer.h"

/**
 * default comment
 */
class T_TX_InterfaceParallel : public T_TX_InterfacePPM
{
  public:
   T_TX_InterfaceParallel();
   virtual ~T_TX_InterfaceParallel();
   
   /**
    * Get input method
    */
   int inputMethod() { return(T_TX_Interface::eIM_parallel); };
   
   /**
    * Initialize interface. Read further config data from a file, if necessary.
    */
   int init(SimpleXMLTransfer* config);
   
   /**
    * Set current input data. If some value is not available, the value 
    * is not overwritten.
    */
   void getInputData(TSimInputs* inputs);

   /**
    * Write configuration back
    */
   virtual void putBackIntoCfg(SimpleXMLTransfer* config);
      
   /** \brief Get the name of the interface for the config file
    *
    *  Returns the name of the interface. It should be a single
    *  word, only lower-case characters.
    */
   const char * getConfigName() {return cname;};

  private:
   void calibrate_parallel_interface();
   int init_parallel_interface(int lpt);
   void get_data_from_parallel_interface(float *rc_channel_values);
   
   void wait_until_interface_is_idle(unsigned char *data);
   void wait_until_data_ready(unsigned char *data);
   void acknowledge_got_data();
   void ask_for_data();
   
   unsigned int LPT_DATA;
   unsigned int LPT_STATUS;
   unsigned int LPT_CONTROL;
     
   /**
    * Number of ticks per second of the mcpu
    */
   float lpt_calibration_factor;
   
   /**
    * Number of parallel port
    */
   int nParPortNum;

   const char *cname;  ///< name in the config file
};

#endif
