#ifndef TX_INTERFACE_RCTRAN_H
#define TX_INTERFACE_RCTRAN_H

#include "../tx_interface.h"
#include "../SimpleXMLTransfer.h"

/**
 * default comment
 */
class T_TX_InterfaceRCTRAN : public T_TX_Interface
{
  public:
   T_TX_InterfaceRCTRAN();
   virtual ~T_TX_InterfaceRCTRAN();
   
   /**
    * Get input method
    */
   virtual int inputMethod() { return(T_TX_Interface::eIM_rctran); };
   
   /**
    * Initialize interface. Read further config data from a file, if necessary.
    */
   int init(SimpleXMLTransfer* config);
   
   /**
    * Write configuration back
    */
   virtual void putBackIntoCfg(SimpleXMLTransfer* config);
      
   /**
    * Set current input data. If some value is not available, the value 
    * is not overwritten.
    */
   void getInputData(TSimInputs* inputs);
   
   /** \brief Get raw interface data as float values.
    *
    *  Fills the memory pointed to by <code>dest</code> with
    *  raw data values in float format
    *  \param dest Memory area to be filled with raw data. Make
    *              sure it can hold getNumAxes() values!
    */
   void getRawData(float *dest);
   
   /**
    * Get the number of axis supported by this interface.
    */
   int getNumAxes();
   
#ifdef WIN32
   unsigned long *rctran;
#else // WIN32
   volatile unsigned long long *rctran;  // Linux LPT rctran interface shmem
#endif
      
  private:
   float cpu_speed;
};

#endif
