#ifndef TX_INTERFACE_RCTRAN2_H
#define TX_INTERFACE_RCTRAN2_H

#include <string>
#include <stdlib.h>

#include "../tx_interfacePPM.h"
#include "../SimpleXMLTransfer.h"

/**
 * This interface reads channel values from a file (default: /proc/rctran).
 * The file contains one line with eight integer values in microseconds, separated
 * by a comma.
 */
class T_TX_Interface_RCTran2 : public T_TX_InterfacePPM
{
  public:
   T_TX_Interface_RCTran2();
   virtual ~T_TX_Interface_RCTran2();
   
   /**
    * Get input method
    */
   int inputMethod() { return(T_TX_Interface::eIM_rctran2); };
   
   /**
    * Initialize interface. Read further config data from a file, if necessary.
    */
   int init(SimpleXMLTransfer* config);
   
   /**
    * Set current input data. If some value is not available, the value 
    * is not overwritten.
    */
   void getInputData(TSimInputs* inputs);

   /**
    * Write configuration back
    */
   virtual void putBackIntoCfg(SimpleXMLTransfer* config);

   /** \brief Get the name of the interface for the config file
    *
    *  Returns the name of the interface. It should be a single
    *  word, only lower-case characters.
    */
   const char * getConfigName() {return cname;};

  private:
   /**
    * Path of the file to read values from
    */
   std::string  filenameToReadFrom;
   
   /**
    * file descriptor
    */
   FILE*        rctran;
  
   const char *cname;  ///< name in the config file
};

#endif
