#ifndef TX_INTERFACE_SOFTWARE_H
# define TX_INTERFACE_SOFTWARE_H

# include "../tx_interface.h"
# include "../SimpleXMLTransfer.h"

/** \brief The software pseudo-Tx interface
 *
 *  This interface handles joysticks, the mouse and the
 *  keyboard as a replacement for a "real" transmitter.
 *  Note: transmitters with a USB interface will also
 *  use this interface as they usually emulate a joystick.
 */
class T_TX_InterfaceSoftware : public T_TX_Interface
{
  public:
    T_TX_InterfaceSoftware(int method);
    virtual ~T_TX_InterfaceSoftware();
    
   /**
    * Get input method
    */
    virtual int inputMethod() { return(input_method); };
    
   /**
    * Initialize interface. Read further config data from a file, if necessary.
    */
    int init(SimpleXMLTransfer* config);
    
   /**
    * Write configuration back
    */
    virtual void putBackIntoCfg(SimpleXMLTransfer* config);
    
   /**
    * Set current input data. If some value is not available, the value
    * is not overwritten.
    */
    void getInputData(TSimInputs* inputs);

   /** \brief Get number of axis.
    * 
    *  Returns the number of axis for the current interface.
    */
    int getNumAxes();

   /** \brief Get raw interface data as float values.
    *
    *  Fills the memory pointed to by <code>dest</code> with
    *  raw data values in float format
    *  \param dest Memory area to be filled with raw data. Make
    *              sure it can hold getNumAxes() values!
    */
    void getRawData(float *dest);

   /**
    * Set the raw value for an axis.
    * \param axis axis number
    * \param x value
    */
    void setAxis(int axis, const float x);
    
   /**
    * Set the controls back to the center. Does not affect throttle!
    */
    void centerControls();
    
    void move_aileron(const float x);
    void move_rudder(const float x);
    void move_elevator(const float x);
    
    void increase_throttle();
    void decrease_throttle();
   
  private:
    float input_raw_data[TX_MAXAXIS];
    float keyb_aileron_input;
    float keyb_rudder_input;
    float keyb_elevator_input;
    float keyb_throttle_input;
    
    int input_method;
};

#endif
