// -*- mode: c; mode: fold -*-
//
//
#include "eom.h"

#include <iostream>
#include <math.h>
#include <cstdlib>

EOM_6DOF::EOM_6DOF()
{
}

EOM_6DOF::EOM_6DOF(CRRCMath::Vector3  initPosEarth,
                   CRRCMath::Vector3  initAng,
                   CRRCMath::Vector3  initVelBody,
                   double             initMass,
                   CRRCMath::Matrix33 initInertia)   /*{{{*/
{
  inertia     = initInertia;
  dMass_inv   = 1.0/initMass;

  dGravity = 9.81; // m/s^2

  if (inertia.det() == 0)
  {
    std::cout << "inertia_inv kann nicht berechnet werden\n";
    exit(-1);
  }
  inertia_inv = inertia.inv();

  // Umrechnung krperfest->erdfest festlegen
  conv.init(initAng);

  // Initialisierung der Geschwindigkeit
  vel.init(initVelBody, CRRCMath::Vector3());

  // Initialisierung der Position
  pos.init(initPosEarth, conv.local(initVelBody));

  // Initialisierung der Drehgeschwindigkeit
  angvel.init(CRRCMath::Vector3(0,0,0), CRRCMath::Vector3(0,0,0));

  initAng.print("euler=", ", ");
  initVelBody.print("v_body=", ", ");
}
/*}}}*/

void EOM_6DOF::setGravity(double iGravity) /*{{{*/
{
  dGravity = iGravity;
}
/*}}}*/

void EOM_6DOF::print(std::string name)     /*{{{*/
{
  std::cout << name;

  conv.updateEuler();

  pos.val.print("posEarth=", ", ");
  vel.val.print("velBody=", ", ");
  angvel.val.print("angvel=", ", ");
  conv.euler.print("euler=", "\n");
}
/*}}}*/

void EOM_6DOF::step(double             dT,
                    CRRCMath::Vector3  FBody,
                    CRRCMath::Vector3  MBody)        /*{{{*/
{
  CRRCMath::Vector3 accel_body;
  CRRCMath::Vector3 angaccel_body;

  
  
  
  // Geschwindigkeit ins erdfeste Koordinatensystem transformieren:
  CRRCMath::Vector3 velEarth = conv.local(vel.val);
  
  // Geschwindigkeit integrieren
  pos.step(dT, velEarth);

  // Drehgeschwindigkeit integrieren
  conv.step(dT, angvel.val);

  
  
  
  
  
  
  // Beschleunigungen addieren:
  //  1. Erdbeschleunigung, umgerechnet ins krperfeste Koordinatensystem
  //  2. Beschleunigung aus FBody
  //  3. der Rotationsterm
  accel_body = conv.body(CRRCMath::Vector3(0, 0, dGravity)) + (FBody*dMass_inv) + (vel.val*angvel.val);

  // Beschleunigung integrieren
  vel.step(dT, accel_body);

  // Drehbeschleunigungen addieren:
  angaccel_body = inertia_inv*( MBody - (angvel.val*(inertia*angvel.val)) );

  // Drehbeschleunigung integrieren
  angvel.step(dT, angaccel_body);

  
  
  
  /*  
  vel.val.print("vel_body=", ", ");
  pos.val.print("pos=", ", ");
  velEarth.print("v_earth=", "\n");
*/
}
/*}}}*/
