#ifndef VALUES_STEP_H
# define VALUES_STEP_H

# include "../math/vector3.h"
# include "../crrc_inputs.h"


namespace Power
{

  /**
   * This class is part of the power system. To simply use the system, you should not
   * access or call any of its members. Please take a look at Power instead.
   *
   * This structure is used to keep low the number of parameters to pass in the power system.
   *
   * @author Jens Wilhelm Wulf
   */
  struct PowerValuesStep
  {
    /**
     * timestep [s]
     */
    double      dt;

    /**
     * pointer to control inputs
     */
    TSimInputs* inputs;

    /**
     * Sums up force created by power system [N] (body axes).
     */
    CRRCMath::Vector3*    force;

    /**
     * Sums up torque created by power system [Nm] (body axes).
     */
    CRRCMath::Vector3*    moment;

    /**
     * Battery voltage [V]
     */
    double      U;
    
    /**
     * Sums up current flowing from the battery [A].
     */
    double      I;
    
    /**
     * Speed of shaft [rad/s].
     */
    double      omega;
    
    /**
     * Sums up torque applied to shaft [Nm].
     */
    double      moment_shaft;
    
    /**
     * Rotational sped of the last propeller calculated [1/s].
     */
    double      dPropFreq;
    
    /**
     * Velocity of airplane body relative to airmass, X-axis [m/s].
     */
    double      V_X;
    
     /**
      * Lowest relative battery capacity left (0..1).
      */
     double  dBatCapLeftMin;
  };
};

#endif
