#include "tx_interfacePPM.h"

T_TX_InterfacePPM::T_TX_InterfacePPM()
{
#if DEBUG_TX_INTERFACE > 0
  printf("T_TX_InterfacePPM::T_TX_InterfacePPM()\n");
#endif  
}

T_TX_InterfacePPM::~T_TX_InterfacePPM()
{
#if DEBUG_TX_INTERFACE > 0
  printf("T_TX_InterfacePPM::~T_TX_InterfacePPM()\n");
#endif 
  delete calib;
  delete map;
}

int T_TX_InterfacePPM::init(SimpleXMLTransfer* config)
{
#if DEBUG_TX_INTERFACE > 0
  printf("int T_TX_InterfacePPM::init(SimpleXMLTransfer* config)\n");
#endif
  std::string name = getConfigName();
  std::string childname = "inputMethod." + name;
  
  calib = new T_Calibration(this, config, childname);
  map   = new T_AxisMapper(this, config, childname);

  return(T_TX_Interface::init(config));
}

void T_TX_InterfacePPM::getInputData(TSimInputs* inputs)
{
#if DEBUG_TX_INTERFACE > 1
  printf("int T_TX_InterfacePPM::getInputData(TSimInputs* inputs)\n");
#endif  
  float throttle = 0.0;
  
  float calibrated;
  int   axisnum;
  
  axisnum    = map->func[ELEVATOR];
  if (axisnum >= 0)
  {
    calibrated = calib->calibrate(axisnum, rc_channel_values[axisnum])
                  * map->inv[ELEVATOR];
    inputs->elevator = mixer->mix_elevator(calibrated);
  }
  
  axisnum    = map->func[AILERON];
  if (axisnum >= 0)
  {
    calibrated = calib->calibrate(axisnum, rc_channel_values[axisnum])
                  * map->inv[AILERON];
    inputs->aileron = mixer->mix_aileron(calibrated);
  }

  axisnum    = map->func[RUDDER];
  if (axisnum >= 0)
  {
    calibrated = calib->calibrate(axisnum, rc_channel_values[axisnum])
                  * map->inv[RUDDER];
    inputs->rudder = mixer->mix_rudder(calibrated);
  }

  axisnum    = map->func[THROTTLE];
  if (axisnum >= 0)
  {
    calibrated = calib->calibrate(axisnum, rc_channel_values[axisnum])
                  * map->inv[THROTTLE];
    throttle = mixer->mix_throttle(calibrated + 0.5);
  }
  
  if (throttle > 0.95)
  {
    inputs->throttle = 1.0;
  }
  else if (throttle < 0.05)
  {
    inputs->throttle = 0.0;
  } 
  else
  {
    inputs->throttle = throttle;
  }
}


void T_TX_InterfacePPM::getRawData(float *dest)
{
  for (int i = 0; i < getNumAxes(); i++)
  {
    *(dest + i) = rc_channel_values[i];
  }
}


void T_TX_InterfacePPM::putBackIntoCfg(SimpleXMLTransfer* config)
{
  calib->putBackIntoCfg(config);
  map->putBackIntoCfg(config);
  
  T_TX_Interface::putBackIntoCfg(config);
}
