#ifndef TX_INTERFACEPPM_H
# define TX_INTERFACEPPM_H

# include "tx_interface.h"
# include "SimpleXMLTransfer.h"

# include <stdio.h>

/**
 * There already are some TX-interfaces implemented into crrcsim and therefore
 * lots of 'if' and things in the code. I wanted to add my own interface, but
 * I don't like 'if' that much, so I implemented a new scheme.
 * All those 'if' could be removed if the interfaces were ported to this scheme.
 * See tx_interface.h and the things in interface_serial2 for an example.
 *
 * Now every other input method uses this scheme, too.
 *
 * There is a pointer to a class T_TX_Interface in crrc_main, which is
 * initialized on startup. Depending on the configuration, it points to one of
 * the interface-classes. That's it, no more 'if'.
 *
 * Jens Wilhelm Wulf, 06.01.2005
 */
class T_TX_InterfacePPM : public T_TX_Interface
{
  public:
   T_TX_InterfacePPM();
   virtual ~T_TX_InterfacePPM();

   /**
    * Initialize interface. Read further config data from a file, if necessary.
    */
   virtual int init(SimpleXMLTransfer* config);

   /**
    * Set current input data. If some value is not available, the value
    * is not overwritten.
    */
   virtual void getInputData(TSimInputs* inputs);

   /** \brief Get raw interface data as float values.
    *
    *  Fills the memory pointed to by <code>dest</code> with
    *  raw data values in float format
    *  \param dest Memory area to be filled with raw data. Make
    *              sure it can hold getNumAxes() values!
    */
   virtual void getRawData(float *dest);
   
   /** \brief Get the name of the interface for the config file
    *
    *  Returns the name of the interface. It should be a single
    *  word, only lower-case characters. Every interface
    *  derived from this class must implement this method!
    */
   virtual const char * getConfigName() = 0;

   /**
    * Write configuration back
    */
   virtual void putBackIntoCfg(SimpleXMLTransfer* config);

  protected:
   /**
    * Channel values, range -1..1
    */
   float rc_channel_values[11];
};

#endif
