#ifndef XMLMODELFILE_H
# define XMLMODELFILE_H

# include "SimpleXMLTransfer.h"

/**
 * This reflects part of the specification of an xml file describing an airplane.
 * 
 * An airplane may contain alternatives as far as 
 *   - graphical representation
 *   - mass (distribution), power system, sound
 * are concerned. This namespace contains methods to tell which grapics and which 
 * 'config' is to be used. This information (user preferences) should not be included 
 * in the file itself! It is written into the in-memory-copy by the application (which 
 * gets it from some user-defined configuration file or whereever).
 * 
 * @author Jens Wilhelm Wulf
 */
namespace XMLModelFile
{
  /**
   * Inserts a list of options to choose from.
   * Returns true if there is something to choose from.
   * 
   * This only copies data from somewhere to somewhere else to make some automatic
   * processing easier.
   *
   * @author Jens Wilhelm Wulf
   */
  bool ListOptions(SimpleXMLTransfer* xml);

  /**
   * Returns the graphics entry to use.
   *
   * @author Jens Wilhelm Wulf
   */
  SimpleXMLTransfer* getGraphics(SimpleXMLTransfer* xml);

  /**
   * Returns the config entry to use.
   *
   * @author Jens Wilhelm Wulf
   */
  SimpleXMLTransfer* getConfig(SimpleXMLTransfer* xml);
   
  /**
   * Returns a pointer to launch presets
   */
  SimpleXMLTransfer* getLaunchPresets(SimpleXMLTransfer* xml);
   
  /**
   * Set index of graphics to use.
   * 
   * @author Jens Wilhelm Wulf
   */
  void SetGraphics(SimpleXMLTransfer* xml, int nIdx);
   
  /**
   * Set index of config to use.
   * 
   * @author Jens Wilhelm Wulf
   */
  void SetConfig(SimpleXMLTransfer* xml, int nIdx);
};
#endif

